#include "StdAfx.h"
#include "AsfCopier.h"
#include "checkhr.h"
#include <vector>
#include <string>
#include <list>
#include <tuple>

using namespace System;
using namespace WMFInterface;
using namespace System::Runtime::InteropServices;
using namespace std;
using namespace tr1;

namespace WMFWrapper
{
	typedef tuple<wstring, wstring, QWORD> scriptData;
	typedef list<scriptData> scriptList;

	struct Source
	{
		CComPtr<IWMSyncReader> reader;
		WORD srcStreamIndex;
		WORD targetStreamNumber;
		WMT_CODEC_INFO_TYPE type;
	};

	void AsfCopier::CopyStreams(array<WMFInterface::SourceInfo^>^ sources, String^ outputFile)
	{
		CComPtr<IWMSyncReader> readers[3];
		int readersCount = 0;
		Source src[3];
		int srcIndex = 0;
		try
		{
			scriptList scriptsList; // 
			for (int i = 0; i < sources->Length; ++i)
			{
				if (sources[i]->CopyScripts)
				{
					CComPtr<IWMSyncReader> tmpReader;
					CHECK_HR(::WMCreateSyncReader(NULL, 0, &tmpReader));
					pin_ptr<const wchar_t> fname = PtrToStringChars(sources[i]->FileName);
					CHECK_HR(tmpReader->Open(fname));
					try
					{
						CComPtr<IWMHeaderInfo> header;
						CHECK_HR(tmpReader->QueryInterface(IID_IWMHeaderInfo, reinterpret_cast<void**>(&header)));
						WORD scripts;
						CHECK_HR(header->GetScriptCount(&scripts));
						for (WORD j = 0; j < scripts; ++j)
						{
							WORD typeLen = 0;
							WORD commandLen = 0;
							QWORD time;
							HRESULT hr = header->GetScript(j, NULL, &typeLen, NULL, &commandLen, &time);
							CHECK_HR(hr);
							vector<WCHAR> typeName(typeLen);
							vector<WCHAR> command(commandLen);
							CHECK_HR(header->GetScript(j, &typeName[0], &typeLen
								, &command[0], &commandLen, &time));
							scriptsList.push_back(scriptData(&typeName[0], &command[0], time));
						}
					}
					finally
					{
						tmpReader->Close();
					}
				}

				array<Stream^>^ tmpStreams = sources[i]->Streams;
				for (int j = 0; j < tmpStreams->Length; ++j)
				{
					CHECK_HR(::WMCreateSyncReader(NULL, 0, &readers[readersCount]));
					pin_ptr<const wchar_t> fname = PtrToStringChars(sources[i]->FileName);
					CHECK_HR(readers[readersCount]->Open(fname));
					++readersCount;

					if (srcIndex >= sizeof(src) / sizeof(Source))
						throw gcnew ArgumentException("Too many source streams");
					src[srcIndex].reader = readers[srcIndex];
					src[srcIndex].srcStreamIndex = tmpStreams[j]->Number;
					switch(tmpStreams[j]->Type)
					{
						case StreamType::Script:
							src[srcIndex].type = WMT_CODECINFO_UNKNOWN;
							break;
						case StreamType::Video:
							src[srcIndex].type = WMT_CODECINFO_VIDEO;
							break;
						case StreamType::Audio:
							src[srcIndex].type = WMT_CODECINFO_AUDIO;
							break;
					}
					src[srcIndex].targetStreamNumber = srcIndex + 1;
					//     
					CHECK_HR(src[srcIndex].reader->SetReadStreamSamples(src[srcIndex].srcStreamIndex, TRUE));
					++srcIndex;
				}
			}
			int streamsCount = srcIndex; // 
			//
			CComPtr<IWMProfileManager> profileManager;
			CHECK_HR(::WMCreateProfileManager(&profileManager));
			CComPtr<IWMProfile> outProfile;
			CHECK_HR(profileManager->CreateEmptyProfile(WMT_VER_9_0, &outProfile));
			//       
			CComPtr<IWMProfile> inputProfiles[3]; //  
			CComPtr<IWMStreamConfig> streamConfig[3]; // 
			CComPtr<IWMHeaderInfo2> headerInfo[3]; //
			for (int i = 0; i < streamsCount; ++i)
			{
				CHECK_HR(src[i].reader->QueryInterface(IID_IWMProfile, reinterpret_cast<void**>(&inputProfiles[i])));
				CHECK_HR(inputProfiles[i]->GetStreamByNumber(src[i].srcStreamIndex, &streamConfig[i]));
				CHECK_HR(streamConfig[i]->SetStreamNumber(src[i].targetStreamNumber));
				CHECK_HR(outProfile->AddStream(streamConfig[i]));
				CHECK_HR(src[i].reader->QueryInterface(IID_IWMHeaderInfo2, reinterpret_cast<void**>(&headerInfo[i])));
			}
			//   writer'
			CComPtr<IWMWriter> writer;
			CHECK_HR(::WMCreateWriter(NULL, &writer));
			CHECK_HR(writer->SetProfile(outProfile));
			pin_ptr<const wchar_t> fname = PtrToStringChars(outputFile);
			CHECK_HR(writer->SetOutputFilename(fname));
			CComPtr<IWMHeaderInfo3> outHeader;
			CHECK_HR(writer->QueryInterface(IID_IWMHeaderInfo3, reinterpret_cast<void**>(&outHeader)));
			CComPtr<IWMWriterAdvanced> advWriter;
			CHECK_HR(writer->QueryInterface(IID_IWMWriterAdvanced, reinterpret_cast<void**>(&advWriter)));
			for (int i = 0; i < streamsCount; ++i)
			{
				CHECK_HR(writer->SetInputProps(i, NULL));
				if (src[i].type == WMT_CODECINFO_UNKNOWN)
					continue;
				DWORD codecNumber;
				CHECK_HR(headerInfo[i]->GetCodecInfoCount(&codecNumber));
				for (DWORD j = 0; j < codecNumber; ++j)
				{
					WORD codecNameLen;
					WORD codecDescLen;
					WMT_CODEC_INFO_TYPE codecType;
					WORD codecInfoLen;
					CHECK_HR(headerInfo[i]->GetCodecInfo(j, &codecNameLen, NULL
						, &codecDescLen, NULL, &codecType, &codecInfoLen, NULL));
					if (codecType != src[i].type)
						continue;
					vector<WCHAR> codecName(codecNameLen);
					vector<WCHAR> codecDesc(codecDescLen);
					vector<BYTE> codecInfo(codecInfoLen);
					CHECK_HR(headerInfo[i]->GetCodecInfo(j, &codecNameLen, &codecName[0]
						, &codecDescLen, &codecDesc[0], &codecType, &codecInfoLen, &codecInfo[0]));
					CHECK_HR(outHeader->AddCodecInfo(&codecName[0], &codecDesc[0], codecType
						, codecInfoLen, &codecInfo[0]));
				}
			}
			// 
			scriptList::const_iterator it1 = scriptsList.begin();
			scriptList::const_iterator it2 = scriptsList.end();
			while (it1 != it2)
			{
				CHECK_HR(outHeader->AddScript(const_cast<WCHAR*>(get<0>(*it1).c_str()), const_cast<WCHAR*>(get<1>(*it1).c_str()), get<2>(*it1)));
				++it1;
			}

			CHECK_HR(writer->BeginWriting());			
			int activeStreamsCount = streamsCount;
			int readerIndicies[3] = { 0, 1, 2};
			QWORD sampleTime[3] = {0, 0, 0};
			int activeReaderIndex = 0;
			for (;;)
			{
				INSSBuffer* buf;
				QWORD duration;
				DWORD flags;
				int readerIndex = readerIndicies[activeReaderIndex];
				const Source& current = src[readerIndex];
				HRESULT hr = current.reader->GetNextSample(current.srcStreamIndex
					, &buf, &sampleTime[readerIndex], &duration, &flags, NULL, NULL);
				if (hr == S_OK)
				{
					
					CHECK_HR(advWriter->WriteStreamSample(current.targetStreamNumber
						, sampleTime[readerIndex], 0, duration
						, flags & (WM_SF_CLEANPOINT | WM_SF_DISCONTINUITY), buf));
					//sampleTime[readerIndex] += duration;
					//  ,       
					if (activeStreamsCount > 1)
					{
						int minIndex;
						if (activeStreamsCount == 3)
						{
							int i1 = (activeReaderIndex + 1) % 3;
							int i2 = (activeReaderIndex + 2) % 3;
							minIndex = sampleTime[readerIndicies[i1]]
											< sampleTime[readerIndicies[i2]]
										? i1 : i2;
						}
						else
							minIndex = (activeReaderIndex + 1) % 2;
						if (sampleTime[readerIndicies[minIndex]] < sampleTime[readerIndex])
							activeReaderIndex = minIndex;
					}
					continue;
				}
				if (hr == NS_E_NO_MORE_SAMPLES) // 
				{
					if (activeStreamsCount > 1)
					{
						int j = activeReaderIndex;
						for (int i = j + 1; i < activeStreamsCount; ++i, ++j)
						{
							readerIndicies[j] = readerIndicies[i];							
						}
						--activeStreamsCount;
						if (activeStreamsCount > 1)
						{
							activeReaderIndex =
								sampleTime[readerIndicies[0]] < sampleTime[readerIndicies[1]]
									? 0 : 1;
						}
						else
							activeReaderIndex = 0;
					}
					else
						break;
					continue;
				}
				Marshal::ThrowExceptionForHR(hr);
			}			
			CHECK_HR(writer->EndWriting());
		}
		finally
		{
			for (int i = 0; i < readersCount; ++i)
			{
				readers[i]->Close();
			}
		}
	}

	void AsfCopier::TransferStream(IWMSyncReader* reader, IWMWriterAdvanced* writer, WORD inStreamNum, WORD outStreamNum)
	{
		INSSBuffer* buf;
		QWORD sampleTime;
		QWORD duration;
		DWORD flags;
		HRESULT hr;
		while (hr = reader->GetNextSample(inStreamNum, &buf, &sampleTime, &duration
			, &flags, NULL, NULL) == S_OK)
		{
			CHECK_HR(writer->WriteStreamSample(outStreamNum, sampleTime, 0, duration
				, flags & (WM_SF_CLEANPOINT | WM_SF_DISCONTINUITY), buf));
		}
		if (hr == NS_E_NO_MORE_SAMPLES) // 
			return;
		Marshal::ThrowExceptionForHR(hr);
	}	
}
